searchData={"content_type":"text/markdown","items":[{"doc":"Functions to convert strings, atoms and map keys between `camelCase`,\n`snake_case` and `kebab-case`.\n\nCurrently this functions do not support UTF-8.\n\nIf this package fits not your requirements then take a look here:\n\n  * [Macro.camelize/1](https://hexdocs.pm/elixir/Macro.html#camelize/1) and\n    [Macro.underscore/1](https://hexdocs.pm/elixir/Macro.html#underscore/1)\n  * [ReCase](https://github.com/sobolevn/recase) helps you to convert a string\n    from any case to any case.\n  * [ProperCase](https://github.com/johnnyji/proper_case) an Elixir library that\n    converts keys in maps between snake_case and camel_case.","ref":"ConvCase.html","title":"ConvCase","type":"module"},{"doc":"Converts `snake_case` and `kebab-case` into `camelCase`.\n\nFor strings, the function returns the converted string.","ref":"ConvCase.html#to_camel_case/1","title":"ConvCase.to_camel_case/1","type":"function"},{"doc":"iex> ConvCase.to_camel_case(\"foo_bar\")\n    \"fooBar\"\n\n    iex> ConvCase.to_camel_case(\"foo-bar\")\n    \"fooBar\"\n\nFor atoms, the function returns the converted atom. This function used\n`String.to_existing_atom/1`.","ref":"ConvCase.html#to_camel_case/1-examples","title":"Examples - ConvCase.to_camel_case/1","type":"function"},{"doc":"iex> ConvCase.to_camel_case(:foo_bar)\n    :fooBar\n\nFor lists, the function returns a list with converted values.","ref":"ConvCase.html#to_camel_case/1-examples","title":"Examples - ConvCase.to_camel_case/1","type":"function"},{"doc":"iex> ConvCase.to_camel_case([\"foo_bar\", \"foo-bar\"])\n    [\"fooBar\", \"fooBar\"]\n\nFor tuples, the function returns a tuple with converted values.","ref":"ConvCase.html#to_camel_case/1-examples","title":"Examples - ConvCase.to_camel_case/1","type":"function"},{"doc":"iex> ConvCase.to_camel_case({\"foo_bar\", \"foo-bar\"})\n    {\"fooBar\", \"fooBar\"}\n\nFor maps, the function returns a map with converted keys. The type of the key\nwill not be changed. New atoms are generated by `String.to_existing_atom/1`.\nKeys of nested maps are converted too.","ref":"ConvCase.html#to_camel_case/1-examples","title":"Examples - ConvCase.to_camel_case/1","type":"function"},{"doc":"iex> ConvCase.to_camel_case(%{foo_bar: %{\"foo-bar\" => \"foo-bar\"}})\n    %{fooBar: %{\"fooBar\" => \"foo-bar\"}}\n\nFor other types, the function returns the given value.","ref":"ConvCase.html#to_camel_case/1-examples","title":"Examples - ConvCase.to_camel_case/1","type":"function"},{"doc":"iex> ConvCase.to_camel_case(42)\n    42","ref":"ConvCase.html#to_camel_case/1-examples","title":"Examples - ConvCase.to_camel_case/1","type":"function"},{"doc":"Converts `snake_case` and `camelCase` into `kebab-case`.\n\nFor strings, the function returns the converted string.","ref":"ConvCase.html#to_kebab_case/1","title":"ConvCase.to_kebab_case/1","type":"function"},{"doc":"iex> ConvCase.to_kebab_case(\"foo_bar\")\n    \"foo-bar\"\n\n    iex> ConvCase.to_kebab_case(\"fooBar\")\n    \"foo-bar\"\n\nFor atoms, the function returns the converted atom. This function used\n`String.to_existing_atom/1`.","ref":"ConvCase.html#to_kebab_case/1-examples","title":"Examples - ConvCase.to_kebab_case/1","type":"function"},{"doc":"iex> ConvCase.to_kebab_case(:foo_bar)\n    :\"foo-bar\"\n\nFor lists, the function returns a list with converted values.","ref":"ConvCase.html#to_kebab_case/1-examples","title":"Examples - ConvCase.to_kebab_case/1","type":"function"},{"doc":"iex> ConvCase.to_kebab_case([\"foo_bar\", \"fooBar\"])\n    [\"foo-bar\", \"foo-bar\"]\n\nFor tuples, the function returns a tuple with converted values.","ref":"ConvCase.html#to_kebab_case/1-examples","title":"Examples - ConvCase.to_kebab_case/1","type":"function"},{"doc":"iex> ConvCase.to_kebab_case({\"foo_bar\", \"fooBar\"})\n    {\"foo-bar\", \"foo-bar\"}\n\nFor maps, the function returns a map with converted keys. The type of the key\nwill not be changed. New atoms are generated by `String.to_existing_atom/1`.\nKeys of nested maps are converted too.","ref":"ConvCase.html#to_kebab_case/1-examples","title":"Examples - ConvCase.to_kebab_case/1","type":"function"},{"doc":"iex> ConvCase.to_kebab_case(%{foo_bar: %{\"fooBar\" => \"fooBar\"}})\n    %{\"foo-bar\": %{\"foo-bar\" => \"fooBar\"}}\n\nFor other types, the function returns the given value.","ref":"ConvCase.html#to_kebab_case/1-examples","title":"Examples - ConvCase.to_kebab_case/1","type":"function"},{"doc":"iex> ConvCase.to_kebab_case(42)\n    42","ref":"ConvCase.html#to_kebab_case/1-examples","title":"Examples - ConvCase.to_kebab_case/1","type":"function"},{"doc":"Converts `camelCase` and `kebab-case` into `snake_case`.\n\nFor strings, the function returns the converted string.","ref":"ConvCase.html#to_snake_case/1","title":"ConvCase.to_snake_case/1","type":"function"},{"doc":"iex> ConvCase.to_snake_case(\"fooBar\")\n    \"foo_bar\"\n\n    iex> ConvCase.to_snake_case(\"foo-bar\")\n    \"foo_bar\"\n\nFor atoms, the function returns the converted atom.\n`String.to_existing_atom/1` is used to create \"new\" atoms.\n\n    iex> ConvCase.to_snake_case(:fooBar)\n    :foo_bar\n\nFor lists, the function returns a list with converted values.\n\n    iex> ConvCase.to_snake_case([\"fooBar\", \"foo-bar\"])\n    [\"foo_bar\", \"foo_bar\"]\n\nFor tuples, the function returns a tuple with converted values.\n\n    iex> ConvCase.to_snake_case({\"fooBar\", \"foo-bar\"})\n    {\"foo_bar\", \"foo_bar\"}\n\nFor maps, the function returns a map with converted keys. The type of the key\nwill not be changed. New atoms are generated by `String.to_existing_atom/1`.\nThe keys of nested maps are also converted.\n\n    iex> ConvCase.to_snake_case(%{fooBar: %{\"foo-bar\" => \"foo-bar\"}})\n    %{foo_bar: %{\"foo_bar\" => \"foo-bar\"}}\n\nFor other types, the function returns the given value.\n\n    iex> ConvCase.to_snake_case(42)\n    42","ref":"ConvCase.html#to_snake_case/1-examples","title":"Examples - ConvCase.to_snake_case/1","type":"function"},{"doc":"","ref":"HL7.html","title":"HL7","type":"module"},{"doc":"","ref":"HL7.html#Composition/1","title":"HL7.Composition/1","type":"macro"},{"doc":"","ref":"HL7.html#Composition/2","title":"HL7.Composition/2","type":"macro"},{"doc":"","ref":"HL7.html#DeviceDefinition/1","title":"HL7.DeviceDefinition/1","type":"macro"},{"doc":"","ref":"HL7.html#DeviceDefinition/2","title":"HL7.DeviceDefinition/2","type":"macro"},{"doc":"","ref":"HL7.html#DeviceDispense/1","title":"HL7.DeviceDispense/1","type":"macro"},{"doc":"","ref":"HL7.html#DeviceDispense/2","title":"HL7.DeviceDispense/2","type":"macro"},{"doc":"","ref":"HL7.html#DeviceRequest/1","title":"HL7.DeviceRequest/1","type":"macro"},{"doc":"","ref":"HL7.html#DeviceRequest/2","title":"HL7.DeviceRequest/2","type":"macro"},{"doc":"","ref":"HL7.html#Location/1","title":"HL7.Location/1","type":"macro"},{"doc":"","ref":"HL7.html#Location/2","title":"HL7.Location/2","type":"macro"},{"doc":"","ref":"HL7.html#Organization/1","title":"HL7.Organization/1","type":"macro"},{"doc":"","ref":"HL7.html#Organization/2","title":"HL7.Organization/2","type":"macro"},{"doc":"","ref":"HL7.html#Patient/1","title":"HL7.Patient/1","type":"macro"},{"doc":"","ref":"HL7.html#Patient/2","title":"HL7.Patient/2","type":"macro"},{"doc":"","ref":"HL7.html#Person/1","title":"HL7.Person/1","type":"macro"},{"doc":"","ref":"HL7.html#Person/2","title":"HL7.Person/2","type":"macro"},{"doc":"","ref":"HL7.html#Section/1","title":"HL7.Section/1","type":"macro"},{"doc":"","ref":"HL7.html#Section/2","title":"HL7.Section/2","type":"macro"},{"doc":"","ref":"HL7.html#schema/0","title":"HL7.schema/0","type":"function"},{"doc":"","ref":"HL7.Application.html","title":"HL7.Application","type":"module"},{"doc":"","ref":"HL7.Application.html#port/0","title":"HL7.Application.port/0","type":"function"},{"doc":"","ref":"HL7.Application.html#start/2","title":"HL7.Application.start/2","type":"function"},{"doc":"","ref":"HL7.Endpoint.html","title":"HL7.Endpoint","type":"module"},{"doc":"","ref":"HL7.Endpoint.html#call/2","title":"HL7.Endpoint.call/2","type":"function"},{"doc":"","ref":"HL7.Endpoint.html#init/1","title":"HL7.Endpoint.init/1","type":"function"},{"doc":"","ref":"HL7.Service.html","title":"HL7.Service","type":"module"},{"doc":"","ref":"HL7.Service.html#delete4/5","title":"HL7.Service.delete4/5","type":"function"},{"doc":"","ref":"HL7.Service.html#diff/1","title":"HL7.Service.diff/1","type":"function"},{"doc":"","ref":"HL7.Service.html#encode/1","title":"HL7.Service.encode/1","type":"function"},{"doc":"","ref":"HL7.Service.html#export/1","title":"HL7.Service.export/1","type":"function"},{"doc":"","ref":"HL7.Service.html#get2/3","title":"HL7.Service.get2/3","type":"function"},{"doc":"","ref":"HL7.Service.html#get4/5","title":"HL7.Service.get4/5","type":"function"},{"doc":"","ref":"HL7.Service.html#history/1","title":"HL7.Service.history/1","type":"function"},{"doc":"","ref":"HL7.Service.html#meta/1","title":"HL7.Service.meta/1","type":"function"},{"doc":"","ref":"HL7.Service.html#metadata/1","title":"HL7.Service.metadata/1","type":"function"},{"doc":"","ref":"HL7.Service.html#post2/3","title":"HL7.Service.post2/3","type":"function"},{"doc":"","ref":"HL7.Service.html#post3/4","title":"HL7.Service.post3/4","type":"function"},{"doc":"","ref":"HL7.Service.html#post4/5","title":"HL7.Service.post4/5","type":"function"},{"doc":"","ref":"HL7.Service.html#post5/6","title":"HL7.Service.post5/6","type":"function"},{"doc":"","ref":"HL7.Service.html#postDiff/1","title":"HL7.Service.postDiff/1","type":"function"},{"doc":"","ref":"HL7.Service.html#postExport/1","title":"HL7.Service.postExport/1","type":"function"},{"doc":"","ref":"HL7.Service.html#postMeta/1","title":"HL7.Service.postMeta/1","type":"function"},{"doc":"","ref":"HL7.Service.html#postRoot/1","title":"HL7.Service.postRoot/1","type":"function"},{"doc":"","ref":"HL7.Service.html#put4/5","title":"HL7.Service.put4/5","type":"function"},{"doc":"","ref":"HL7.Service.html#reindex/1","title":"HL7.Service.reindex/1","type":"function"},{"doc":"","ref":"HL7.Service.html#root/1","title":"HL7.Service.root/1","type":"function"},{"doc":"","ref":"HL7.Validation.html","title":"HL7.Validation","type":"module"},{"doc":"","ref":"HL7.Validation.html#cache/0","title":"HL7.Validation.cache/0","type":"function"},{"doc":"","ref":"HL7.Validation.html#suite/0","title":"HL7.Validation.suite/0","type":"function"},{"doc":"","ref":"HL7.Validation.html#test/0","title":"HL7.Validation.test/0","type":"function"},{"doc":"","ref":"HL7.Validation.html#testItem/1","title":"HL7.Validation.testItem/1","type":"function"},{"doc":"A schema validator inspired by [JSON Schema](http://json-schema.org).\n\nAll available keywords to construct a schema are described on page\n[Usage](usage.html).\n\nThis module can be used to construct a schema module. Should a module\ncontain multiple schemas the option `multi: true` is required.\n\n`use Xema` imports `Xema.Builder` and extends the module with the functions\n+ `__MODULE__.valid?/2`\n+ `__MODULE__.validate/2`\n+ `__MODULE__.validate!/2`\n+ `__MODULE__.cast/2`\n+ `__MODULE__.cast!/2`\n+ `__MODULE__.xema/1`\n\nThe macro `xema/2` supports the construction of a schema. After that\nthe schema is available via the functions above.\n\nIn a multi schema module a schema can be tagged with the option\n`default: :schema_name` and then called by\n+ `__MODULE__.valid?/1`\n+ `__MODULE__.validate/1`\n+ `__MODULE__.validate!/1`\n+ `__MODULE__.cast/1`\n+ `__MODULE__.cast!/1`\n+ `__MODULE__.xema/0`\n\nThe functions with arity 1 are also available for single schema modules.\n\nThe macro `xema_struct/1` creates a schema with the coresponding struct.","ref":"Xema.html","title":"Xema","type":"module"},{"doc":"Single schema module:\n\n    iex> defmodule SingleSchema do\n    ...>   use Xema\n    ...>\n    ...>   # The name :num is optional.\n    ...>   xema :num, do: number(minimum: 1)\n    ...> end\n    iex>\n    iex> SingleSchema.valid?(:num, 6)\n    true\n    iex> SingleSchema.valid?(5)\n    true\n    iex> SingleSchema.validate(0)\n    {:error, %Xema.ValidationError{\n       reason: %{minimum: 1, value: 0}\n    }}\n    iex> SingleSchema.cast(\"5\")\n    {:ok, 5}\n    iex> SingleSchema.cast(\"-5\")\n    {:error, %Xema.ValidationError{\n       reason: %{minimum: 1, value: -5}\n    }}\n\nMulti schema module:\n\n    iex> defmodule Schema do\n    ...>   use Xema, multi: true, default: :user\n    ...>\n    ...>   @pos integer(minimum: 0)\n    ...>   @neg integer(maximum: 0)\n    ...>\n    ...>   xema :user do\n    ...>     map(\n    ...>       properties: %{\n    ...>         name: string(min_length: 1),\n    ...>         age: @pos\n    ...>       }\n    ...>     )\n    ...>   end\n    ...>\n    ...>   xema :nums do\n    ...>     map(\n    ...>       properties: %{\n    ...>         pos: list(items: @pos),\n    ...>         neg: list(items: @neg)\n    ...>       }\n    ...>     )\n    ...>   end\n    ...> end\n    iex>\n    iex> Schema.valid?(:user, %{name: \"John\", age: 21})\n    true\n    iex> Schema.valid?(%{name: \"John\", age: 21})\n    true\n    iex> Schema.valid?(%{name: \"\", age: 21})\n    false\n    iex> Schema.validate(%{name: \"John\", age: 21})\n    :ok\n    iex> Schema.validate(%{name: \"\", age: 21})\n    {:error, %Xema.ValidationError{\n      reason: %{\n        properties: %{name: %{min_length: 1, value: \"\"}}}\n      }\n    }\n    iex> Schema.valid?(:nums, %{pos: [1, 2, 3]})\n    true\n    iex> Schema.valid?(:nums, %{neg: [1, 2, 3]})\n    false\n\nStruct schema module:\n\n    iex> defmodule StructA do\n    ...>   use Xema\n    ...>\n    ...>   xema_struct do\n    ...>     field :foo, :integer, minimum: 0\n    ...>   end\n    ...> end\n    ...>\n    ...> defmodule StructB do\n    ...>   use Xema\n    ...>\n    ...>   xema_struct do\n    ...>     field :a, :string, min_length: 3\n    ...>     field :b, StructA\n    ...>     required [:a]\n    ...>   end\n    ...> end\n    ...>\n    ...> data = StructB.cast!(a: \"abc\", b: %{foo: 5})\n    ...> data.a\n    \"abc\"\n    iex> Map.from_struct(data.b)\n    %{foo: 5}\n\nFor more examples to construct schemas see \"[Examples](examples.html)\".","ref":"Xema.html#module-examples","title":"Examples - Xema","type":"module"},{"doc":"Converts the given data using the specified schema. Returns `{:ok, result}` or\n`{:error, reason}`. The `result` is converted and validated with the schema.","ref":"Xema.html#cast/3","title":"Xema.cast/3","type":"function"},{"doc":"iex> schema = Xema.new({:integer, minimum: 1})\n    iex> Xema.cast(schema, \"5\")\n    {:ok, 5}\n    iex> Xema.cast(schema, \"five\")\n    {:error, %Xema.CastError{\n      key: nil,\n      path: [],\n      to: :integer,\n      value: \"five\"\n    }}\n    iex> Xema.cast(schema, \"0\")\n    {:error, %Xema.ValidationError{\n      reason: %{minimum: 1, value: 0}\n    }}","ref":"Xema.html#cast/3-examples","title":"Examples: - Xema.cast/3","type":"function"},{"doc":"If for a value multiple types are defined the function used the result of the\nfirst successful conversion.","ref":"Xema.html#cast/3-multiple-types","title":"Multiple types - Xema.cast/3","type":"function"},{"doc":"iex> schema = Xema.new([:integer, :string, nil])\n    iex> Xema.cast(schema, 5)\n    {:ok, 5}\n    iex> Xema.cast(schema, 5.5)\n    {:ok, \"5.5\"}\n    iex> Xema.cast(schema, \"5\")\n    {:ok, 5}\n    iex> Xema.cast(schema, \"five\")\n    {:ok, \"five\"}\n    iex> Xema.cast(schema, nil)\n    {:ok, nil}\n    iex> Xema.cast(schema, [5])\n    {:error,\n      %Xema.CastError{path: [], to: [:integer, :string, nil], value: [5]}\n    }","ref":"Xema.html#cast/3-examples","title":"Examples - Xema.cast/3","type":"function"},{"doc":"Schemas in a combiner will be cast independently one by one in reverse order.","ref":"Xema.html#cast/3-cast-with-any_of-all_of-and-one_of","title":"Cast with `any_of`, `all_of`, and `one_of` - Xema.cast/3","type":"function"},{"doc":"iex> schema = Xema.new(any_of: [\n    ...>   [properties: %{a: :integer}],\n    ...>   [properties: %{a: :string}]\n    ...> ])\n    iex> Xema.cast(schema, %{a: 5})\n    {:ok, %{a: 5}}\n    iex> Xema.cast(schema, %{a: 5.5})\n    {:ok, %{a: \"5.5\"}}\n    iex> Xema.cast(schema, %{a: \"5\"})\n    {:ok, %{a: 5}}\n    iex> Xema.cast(schema, %{a: \"five\"})\n    {:ok, %{a: \"five\"}}\n    iex> Xema.cast(schema, %{a: [5]})\n    {:error,\n      %Xema.CastError{\n        error: nil,\n        key: nil,\n        message: nil,\n        path: [],\n        to: [\n          %{path: [:a], to: :integer, value: [5]},\n          %{path: [:a], to: :string, value: [5]}\n        ],\n        value: %{a: [5]}\n    }}","ref":"Xema.html#cast/3-examples","title":"Examples - Xema.cast/3","type":"function"},{"doc":"With the option `additional_properties: :delete` additional properties will be\ndeleted on cast. Additional properties will be deleted in schemas with\n`additional_properties: false`.","ref":"Xema.html#cast/3-options","title":"Options - Xema.cast/3","type":"function"},{"doc":"iex> schema = Xema.new(\n    ...>   properties: %{\n    ...>     a: [\n    ...>       properties: %{\n    ...>         foo: :integer\n    ...>       },\n    ...>       additional_properties: false\n    ...>     ],\n    ...>     b: [\n    ...>       properties: %{\n    ...>         foo: :integer\n    ...>       }\n    ...>     ]\n    ...>   }\n    ...> )\n    iex>\n    iex> Xema.cast(schema, %{\n    ...>   a: %{foo: \"6\", bar: \"7\"},\n    ...>   b: %{foo: \"6\", bar: \"7\"},\n    ...> }, additional_properties: :delete)\n    {:ok, %{\n      a: %{foo: 6},\n      b: %{foo: 6, bar: \"7\"}\n    }}","ref":"Xema.html#cast/3-examples","title":"Examples - Xema.cast/3","type":"function"},{"doc":"Converts the given data using the specified schema. Returns the converted data\nor an exception.","ref":"Xema.html#cast!/3","title":"Xema.cast!/3","type":"function"},{"doc":"Creates a `Xema` from a JSON Schema.  The argument `json_schema` is expected\nas a decoded JSON Schema.\n\nAll keys that are not standard JSON Schema keywords have to be known atoms. If\nthe schema has additional keys that are unknown atoms the option\n`atom: :force` is needed. In this case the atoms will be created. This is not\nneeded for keys expected by JSON Schema (e.g. in properties)\n\nOptions:\n* `:draft` specifies the draft to check the given JSON Schema. Possible values\n  are `\"draft4\"`, `\"draft6\"`, and `\"draft7\"`, default is `\"draft7\"`. If\n  `:draft` not set and the schema contains `$schema` then the value for\n  `$schema` is used for this option.\n* `:atoms` creates atoms for unknown atoms when set to `:force`. This is just\n  needed for additional JSON Schema keywords.","ref":"Xema.html#from_json_schema/2","title":"Xema.from_json_schema/2","type":"function"},{"doc":"iex> Xema.from_json_schema(%{\"type\" => \"integer\", \"minimum\" => 5})\n    %Xema{schema: %Xema.Schema{minimum: 5, type: :integer}}\n\n    iex> schema = %{\n    ...>   \"type\" => \"object\",\n    ...>   \"properties\" => %{\"foo\" => %{\"type\" => \"integer\"}}\n    ...> }\n    iex> Xema.from_json_schema(schema)\n    %Xema{schema:\n      %Xema.Schema{\n        properties: %{\"foo\" => %Xema.Schema{type: :integer}},\n        type: :map,\n        keys: :strings\n      }\n    }\n\n    iex> Xema.from_json_schema(%{\"type\" => \"integer\", \"foo\" => \"bar\"}, atom: :force)\n    %Xema{schema: %Xema.Schema{data: %{foo: \"bar\"}, type: :integer}}\n\n    iex> Xema.from_json_schema(%{\"exclusiveMaximum\" => 5}, draft: \"draft7\")\n    %Xema{schema: %Xema.Schema{exclusive_maximum: 5}}\n\n    iex> Xema.from_json_schema(%{\"exclusiveMaximum\" => 5}, draft: \"draft4\")\n    ** (Xema.SchemaError) Can't build schema:\n    Dependencies for \"exclusiveMaximum\" failed. Missing required key \"maximum\".","ref":"Xema.html#from_json_schema/2-examples","title":"Examples - Xema.from_json_schema/2","type":"function"},{"doc":"This function creates the schema from the given `data`.\n\nPossible options:\n+ `:loader` - a loader for remote schemas. This option will overwrite the\n              loader from the config.\n              See [Configure a loader](loader.html) to how to define a loader.\n\n+ `inline` - inlined all references in the schema. Default `:true`.","ref":"Xema.html#new/2","title":"Xema.new/2","type":"function"},{"doc":"Simple schema:\n\n    iex> schema = Xema.new :string\n    iex> Xema.valid? schema, \"hello\"\n    true\n    iex> Xema.valid? schema, 42\n    false\n\nSchema:\n\n    iex> schema = Xema.new {:string, min_length: 3, max_length: 12}\n    iex> Xema.valid? schema, \"hello\"\n    true\n    iex> Xema.valid? schema, \"hi\"\n    false\n\nNested schemas:\n\n    iex> schema = Xema.new {:list, items: {:number, minimum: 2}}\n    iex> Xema.validate(schema, [2, 3, 4])\n    :ok\n    iex> Xema.valid?(schema, [2, 3, 4])\n    true\n    iex> Xema.validate(schema, [2, 3, 1])\n    {:error, %Xema.ValidationError{\n      reason: %{\n        items: %{2 => %{value: 1, minimum: 2}}}\n      }\n    }\n\nMore examples can be found on page\n[Usage](https://hexdocs.pm/xema/usage.html#content).","ref":"Xema.html#new/2-examples","title":"Examples - Xema.new/2","type":"function"},{"doc":"Returns the source for a given `xema`. The output can differ from the input\nif the schema contains references. To get the original source the schema\nmust be created with `inline: false`.","ref":"Xema.html#source/1","title":"Xema.source/1","type":"function"},{"doc":"iex> {:integer, minimum: 1} |> Xema.new() |> Xema.source()\n    {:integer, minimum: 1}","ref":"Xema.html#source/1-examples","title":"Examples - Xema.source/1","type":"function"},{"doc":"Returns `true` if the `value` is a valid value against the given `schema`;\notherwise returns `false`.","ref":"Xema.html#valid?/2","title":"Xema.valid?/2","type":"function"},{"doc":"Returns `:ok` if the `value` is a valid value against the given `schema`;\notherwise returns an error tuple.\n\nWith the option `:fail`, you can define when the validation is aborted. This\nalso influences how many error reasons are returned.\n- `:immediately` aborts the validation when the first validation fails.\n- `:early` (default) aborts on failed validations, but runs validations\n  for all properties and items.\n- `:finally` aborts after all possible validations.","ref":"Xema.html#validate/3","title":"Xema.validate/3","type":"function"},{"doc":"Returns `:ok` if the `value` is a valid value against the given `schema`;\notherwise raises a `Elixir.Xema.ValidationError`. See `validate3` for\navailable options.","ref":"Xema.html#validate!/3","title":"Xema.validate!/3","type":"function"},{"doc":"The return type of a validation run.","ref":"Xema.html#t:result/0","title":"Xema.result/0","type":"type"},{"doc":"","ref":"Xema.html#t:schema/0","title":"Xema.schema/0","type":"type"},{"doc":"This struct contains the schema and references of the schema.","ref":"Xema.html#t:t/0","title":"Xema.t/0","type":"type"},{"doc":"A behaviour module for implementing a schema validator. This behaviour is\njust for `Xema` and `JsonXema`.","ref":"Xema.Behaviour.html","title":"Xema.Behaviour","type":"behaviour"},{"doc":"This callback initialize the schema. The function gets the data given to\n`Xema.new/1` and returns a `Xema.Schema`.","ref":"Xema.Behaviour.html#c:init/2","title":"Xema.Behaviour.init/2","type":"callback"},{"doc":"The schema container.","ref":"Xema.Behaviour.html#t:t/0","title":"Xema.Behaviour.t/0","type":"type"},{"doc":"This module contains some convenience functions to generate schemas. Beside\nthe type-functions the module contains the combinator-functions `all_of/2`,\n`any_of/2` and `one_of/2`.","ref":"Xema.Builder.html","title":"Xema.Builder","type":"module"},{"doc":"iex> import Xema.Builder\n    ...> schema = Xema.new integer(minimum: 1)\n    ...> Xema.valid?(schema, 6)\n    true\n    ...> Xema.valid?(schema, 0)\n    false","ref":"Xema.Builder.html#module-examples","title":"Examples - Xema.Builder","type":"module"},{"doc":"Returns a tuple with the given `type` (default `:any`) and the given schemas\ntagged by the keyword `:all_of`. This function provides a shortcut for\nsomething like `integer(all_of: [...])` or `any(all_of: [...])`.","ref":"Xema.Builder.html#all_of/2","title":"Xema.Builder.all_of/2","type":"function"},{"doc":"iex> Xema.Builder.all_of([:integer, :string])\n    {:any, all_of: [:integer, :string] }\n\n    iex> Xema.Builder.all_of(:integer, [[minimum: 10], [maximum: 5]])\n    {:integer, all_of: [[minimum: 10], [maximum: 5]]}\n\n```elixir\ndefmodule MySchema do\n  use Xema\n\n  xema do\n    all_of [\n      list(items: integer(minimum: 1, maximum: 66)),\n      list(items: integer(minimum: 33, maximum: 100))\n    ]\n  end\nend\n\nMySchema.valid?([20, 30]) #=> false\nMySchema.valid?([40, 50]) #=> true\nMySchema.valid?([60, 70]) #=> false\nMySchema.valid?([10, 90]) #=> false\n```","ref":"Xema.Builder.html#all_of/2-examples","title":"Examples - Xema.Builder.all_of/2","type":"function"},{"doc":"Returns :any.","ref":"Xema.Builder.html#any/0","title":"Xema.Builder.any/0","type":"function"},{"doc":"Returns a tuple of `:any` and the given keyword list.","ref":"Xema.Builder.html#any/1","title":"Xema.Builder.any/1","type":"function"},{"doc":"iex> Xema.Builder.any(key: 42)\n    {:any, key: 42}","ref":"Xema.Builder.html#any/1-examples","title":"Examples - Xema.Builder.any/1","type":"function"},{"doc":"Returns a tuple with the given `type` (default `:any`) and the given schemas\ntagged by the keyword `:any_of`. This function provides a shortcut for\nsomething like `integer(any_of: [...])` or `any(any_of: [...])`.","ref":"Xema.Builder.html#any_of/2","title":"Xema.Builder.any_of/2","type":"function"},{"doc":"iex> Xema.Builder.any_of([:integer, :string])\n    {:any, any_of: [:integer, :string] }\n\n    iex> Xema.Builder.any_of(:integer, [[minimum: 10], [maximum: 5]])\n    {:integer, any_of: [[minimum: 10], [maximum: 5]]}\n\n```elixir\ndefmodule MySchema do\n  use Xema\n\n  xema do\n    any_of [\n      list(items: integer(minimum: 1, maximum: 66)),\n      list(items: integer(minimum: 33, maximum: 100))\n    ]\n  end\nend\n\nMySchema.valid?([20, 30]) #=> true\nMySchema.valid?([40, 50]) #=> true\nMySchema.valid?([60, 70]) #=> true\nMySchema.valid?([10, 90]) #=> false\n```","ref":"Xema.Builder.html#any_of/2-examples","title":"Examples - Xema.Builder.any_of/2","type":"function"},{"doc":"Returns :atom.","ref":"Xema.Builder.html#atom/0","title":"Xema.Builder.atom/0","type":"function"},{"doc":"Returns a tuple of `:atom` and the given keyword list.","ref":"Xema.Builder.html#atom/1","title":"Xema.Builder.atom/1","type":"function"},{"doc":"iex> Xema.Builder.atom(key: 42)\n    {:atom, key: 42}","ref":"Xema.Builder.html#atom/1-examples","title":"Examples - Xema.Builder.atom/1","type":"function"},{"doc":"Returns :boolean.","ref":"Xema.Builder.html#boolean/0","title":"Xema.Builder.boolean/0","type":"function"},{"doc":"Returns a tuple of `:boolean` and the given keyword list.","ref":"Xema.Builder.html#boolean/1","title":"Xema.Builder.boolean/1","type":"function"},{"doc":"iex> Xema.Builder.boolean(key: 42)\n    {:boolean, key: 42}","ref":"Xema.Builder.html#boolean/1-examples","title":"Examples - Xema.Builder.boolean/1","type":"function"},{"doc":"Specifies a field. This function will be used inside `xema/0`.\n\nArguments:\n\n+ `name`: the name of the field.\n+ `type`: the type of the field. The `type` can also be a `struct` or another\n   schema.\n+ `opts`: the rules for the field.","ref":"Xema.Builder.html#field/3","title":"Xema.Builder.field/3","type":"function"},{"doc":"iex> defmodule User do\n    ...>   use Xema\n    ...>\n    ...>   xema_struct do\n    ...>     field :name, :string, min_length: 1\n    ...>   end\n    ...> end\n    ...>\n    iex> %{\"name\" => \"Tim\"} |> User.cast!() |> Map.from_struct()\n    %{name: \"Tim\"}\n\nFor more examples see \"[Examples: Struct](examples.html#struct)\".","ref":"Xema.Builder.html#field/3-examples","title":"Examples - Xema.Builder.field/3","type":"function"},{"doc":"Returns :float.","ref":"Xema.Builder.html#float/0","title":"Xema.Builder.float/0","type":"function"},{"doc":"Returns a tuple of `:float` and the given keyword list.","ref":"Xema.Builder.html#float/1","title":"Xema.Builder.float/1","type":"function"},{"doc":"iex> Xema.Builder.float(key: 42)\n    {:float, key: 42}","ref":"Xema.Builder.html#float/1-examples","title":"Examples - Xema.Builder.float/1","type":"function"},{"doc":"Returns :integer.","ref":"Xema.Builder.html#integer/0","title":"Xema.Builder.integer/0","type":"function"},{"doc":"Returns a tuple of `:integer` and the given keyword list.","ref":"Xema.Builder.html#integer/1","title":"Xema.Builder.integer/1","type":"function"},{"doc":"iex> Xema.Builder.integer(key: 42)\n    {:integer, key: 42}","ref":"Xema.Builder.html#integer/1-examples","title":"Examples - Xema.Builder.integer/1","type":"function"},{"doc":"Returns :keyword.","ref":"Xema.Builder.html#keyword/0","title":"Xema.Builder.keyword/0","type":"function"},{"doc":"Returns a tuple of `:keyword` and the given keyword list.","ref":"Xema.Builder.html#keyword/1","title":"Xema.Builder.keyword/1","type":"function"},{"doc":"iex> Xema.Builder.keyword(key: 42)\n    {:keyword, key: 42}","ref":"Xema.Builder.html#keyword/1-examples","title":"Examples - Xema.Builder.keyword/1","type":"function"},{"doc":"Returns :list.","ref":"Xema.Builder.html#list/0","title":"Xema.Builder.list/0","type":"function"},{"doc":"Returns a tuple of `:list` and the given keyword list.","ref":"Xema.Builder.html#list/1","title":"Xema.Builder.list/1","type":"function"},{"doc":"iex> Xema.Builder.list(key: 42)\n    {:list, key: 42}","ref":"Xema.Builder.html#list/1-examples","title":"Examples - Xema.Builder.list/1","type":"function"},{"doc":"Returns :map.","ref":"Xema.Builder.html#map/0","title":"Xema.Builder.map/0","type":"function"},{"doc":"Returns a tuple of `:map` and the given keyword list.","ref":"Xema.Builder.html#map/1","title":"Xema.Builder.map/1","type":"function"},{"doc":"iex> Xema.Builder.map(key: 42)\n    {:map, key: 42}","ref":"Xema.Builder.html#map/1-examples","title":"Examples - Xema.Builder.map/1","type":"function"},{"doc":"Returns :number.","ref":"Xema.Builder.html#number/0","title":"Xema.Builder.number/0","type":"function"},{"doc":"Returns a tuple of `:number` and the given keyword list.","ref":"Xema.Builder.html#number/1","title":"Xema.Builder.number/1","type":"function"},{"doc":"iex> Xema.Builder.number(key: 42)\n    {:number, key: 42}","ref":"Xema.Builder.html#number/1-examples","title":"Examples - Xema.Builder.number/1","type":"function"},{"doc":"Returns a tuple with the given `type` (default `:any`) and the given schemas\ntagged by the keyword `:one_of`. This function provides a shortcut for\nsomething like `integer(one_of: [...])` or `any(one_of: [...])`.","ref":"Xema.Builder.html#one_of/2","title":"Xema.Builder.one_of/2","type":"function"},{"doc":"iex> Xema.Builder.one_of([:integer, :string])\n    {:any, one_of: [:integer, :string] }\n\n    iex> Xema.Builder.one_of(:integer, [[minimum: 10], [maximum: 5]])\n    {:integer, one_of: [[minimum: 10], [maximum: 5]]}\n\n```elixir\ndefmodule MySchema do\n  use Xema\n\n  xema do\n    one_of [\n      list(items: integer(minimum: 1, maximum: 66)),\n      list(items: integer(minimum: 33, maximum: 100))\n    ]\n  end\nend\n\nMySchema.valid?([20, 30]) #=> true\nMySchema.valid?([40, 50]) #=> false\nMySchema.valid?([60, 70]) #=> true\nMySchema.valid?([10, 90]) #=> false\n```","ref":"Xema.Builder.html#one_of/2-examples","title":"Examples - Xema.Builder.one_of/2","type":"function"},{"doc":"Returns the tuple `{:ref, ref}`.","ref":"Xema.Builder.html#ref/1","title":"Xema.Builder.ref/1","type":"function"},{"doc":"Sets the list of required fields.  Specifies a field. This function will be\nused inside `xema/0`.","ref":"Xema.Builder.html#required/1","title":"Xema.Builder.required/1","type":"function"},{"doc":"iex> defmodule Person do\n    ...>   use Xema\n    ...>\n    ...>   xema_struct do\n    ...>     field :name, :string, min_length: 1\n    ...>     required [:name]\n    ...>   end\n    ...> end\n    ...>\n    iex> %{\"name\" => \"Tim\"} |> Person.cast!() |> Map.from_struct()\n    %{name: \"Tim\"}","ref":"Xema.Builder.html#required/1-examples","title":"Examples - Xema.Builder.required/1","type":"function"},{"doc":"Returns :string.","ref":"Xema.Builder.html#string/0","title":"Xema.Builder.string/0","type":"function"},{"doc":"Returns a tuple of `:string` and the given keyword list.","ref":"Xema.Builder.html#string/1","title":"Xema.Builder.string/1","type":"function"},{"doc":"iex> Xema.Builder.string(key: 42)\n    {:string, key: 42}","ref":"Xema.Builder.html#string/1-examples","title":"Examples - Xema.Builder.string/1","type":"function"},{"doc":"Returns `:struct`.","ref":"Xema.Builder.html#strux/0","title":"Xema.Builder.strux/0","type":"function"},{"doc":"Returns a tuple of `:struct` and the given keyword list when the function gets\na keyword list.\n\nReturns the tuple `{:struct, module: module}` when the function gets an atom.","ref":"Xema.Builder.html#strux/1","title":"Xema.Builder.strux/1","type":"function"},{"doc":"","ref":"Xema.Builder.html#strux/2","title":"Xema.Builder.strux/2","type":"function"},{"doc":"Returns :tuple.","ref":"Xema.Builder.html#tuple/0","title":"Xema.Builder.tuple/0","type":"function"},{"doc":"Returns a tuple of `:tuple` and the given keyword list.","ref":"Xema.Builder.html#tuple/1","title":"Xema.Builder.tuple/1","type":"function"},{"doc":"iex> Xema.Builder.tuple(key: 42)\n    {:tuple, key: 42}","ref":"Xema.Builder.html#tuple/1-examples","title":"Examples - Xema.Builder.tuple/1","type":"function"},{"doc":"Creates a `schema`.","ref":"Xema.Builder.html#xema/1","title":"Xema.Builder.xema/1","type":"macro"},{"doc":"Creates a `schema` with the given name.","ref":"Xema.Builder.html#xema/2","title":"Xema.Builder.xema/2","type":"macro"},{"doc":"Creates a `schema` and defines the corresponding struct.","ref":"Xema.Builder.html#xema_struct/1","title":"Xema.Builder.xema_struct/1","type":"macro"},{"doc":"Converts data using the specified schema.","ref":"Xema.Castable.html","title":"Xema.Castable","type":"protocol"},{"doc":"Converts the given data using the specified schema.","ref":"Xema.Castable.html#cast/2","title":"Xema.Castable.cast/2","type":"function"},{"doc":"","ref":"Xema.Castable.html#t:t/0","title":"Xema.Castable.t/0","type":"type"},{"doc":"A behaviour for a caster.","ref":"Xema.Caster.html","title":"Xema.Caster","type":"behaviour"},{"doc":"A callback for a caster.","ref":"Xema.Caster.html#c:cast/1","title":"Xema.Caster.cast/1","type":"callback"},{"doc":"This module contains semantic validators for strings.","ref":"Xema.Format.html","title":"Xema.Format","type":"module"},{"doc":"Checks if the `string` is a valid date representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 3339](https://tools.ietf.org/html/rfc3339), `false` otherwise.","ref":"Xema.Format.html#date?/1","title":"Xema.Format.date?/1","type":"function"},{"doc":"Checks if the `string` is a valid date time representation.\n\nThis function returns `true` if the value is a `string` and is formatted as\ndefined by [RFC 3339](https://tools.ietf.org/html/rfc3339), `false` otherwise.","ref":"Xema.Format.html#date_time?/1","title":"Xema.Format.date_time?/1","type":"function"},{"doc":"Checks if the `string` is a valid email representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 5322](https://tools.ietf.org/html/rfc5322), `false` otherwise.\n\nThe regular expression was taken from\n[https://emailregex.com/](https://emailregex.com/).","ref":"Xema.Format.html#email?/1","title":"Xema.Format.email?/1","type":"function"},{"doc":"iex> import Xema.Format\n    iex>\n    iex> email?(\"marin.musterman@germany.net\")\n    true\n    iex> email?(\"Otto.Normalverbraucher\")\n    false\n    iex> email?(\"Otto.Normal@Verbraucher.NET\")\n    true","ref":"Xema.Format.html#email?/1-examples","title":"Examples - Xema.Format.email?/1","type":"function"},{"doc":"Checks if the `string` is a valid host representation.\n\nThis function returns `true` if the value is a valid IPv4 address, IPv6\naddress, or a valid hostname, `false` otherwise.","ref":"Xema.Format.html#host?/1","title":"Xema.Format.host?/1","type":"function"},{"doc":"iex> import Xema.Format\n    iex>\n    iex> host?(\"127.0.0.1\")\n    true\n    iex> host?(\"localhost\")\n    true\n    iex> host?(\"elixirforum.com\")\n    true\n    iex> host?(\"go go go\")\n    false","ref":"Xema.Format.html#host?/1-examples","title":"Examples - Xema.Format.host?/1","type":"function"},{"doc":"Checks if the `string` is a valid hostname representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 1034](https://tools.ietf.org/html/rfc1034), `false` otherwise.","ref":"Xema.Format.html#hostname?/1","title":"Xema.Format.hostname?/1","type":"function"},{"doc":"Checks if the `string` is a valid IPv4 address representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 2673](https://tools.ietf.org/html/rfc2673), `false` otherwise.","ref":"Xema.Format.html#ipv4?/1","title":"Xema.Format.ipv4?/1","type":"function"},{"doc":"Checks if the `string` is a valid IPv6 address representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 2373](https://tools.ietf.org/html/rfc2373), `false` otherwise.","ref":"Xema.Format.html#ipv6?/1","title":"Xema.Format.ipv6?/1","type":"function"},{"doc":"Checks if the value matches the given type. The function expected a available\n`format` and a `string` to check. Returns `true` for a valid `string`, `false`\notherwise.","ref":"Xema.Format.html#is?/2","title":"Xema.Format.is?/2","type":"function"},{"doc":"iex> Xema.Format.is?(:email, \"foo@bar.net\")\n    true\n    iex> Xema.Format.is?(:email, \"foo.bar.net\")\n    false","ref":"Xema.Format.html#is?/2-examples","title":"Examples - Xema.Format.is?/2","type":"function"},{"doc":"Checks if the `string` is a valid JSON pointer representation.","ref":"Xema.Format.html#json_pointer?/1","title":"Xema.Format.json_pointer?/1","type":"function"},{"doc":"Return true if `string` contains a regular expression.","ref":"Xema.Format.html#regex?/1","title":"Xema.Format.regex?/1","type":"function"},{"doc":"Checks if the `string` is a valid JSON pointer representation.","ref":"Xema.Format.html#relative_json_pointer?/1","title":"Xema.Format.relative_json_pointer?/1","type":"function"},{"doc":"Returns `true` for a valid format type, `false` otherwise.  This macro can be\nused in guards.","ref":"Xema.Format.html#supports/1","title":"Xema.Format.supports/1","type":"macro"},{"doc":"Checks if the `string` is a valid time representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 3339](https://tools.ietf.org/html/rfc3339), `false` otherwise.","ref":"Xema.Format.html#time?/1","title":"Xema.Format.time?/1","type":"function"},{"doc":"Checks if the `string` is a valid URI representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 3986](https://tools.ietf.org/html/rfc3986), `false` otherwise.\n\nThe following are two example URIs and their component parts:\n```code\n                      hierarchical part\n                            |\n        |-----------------------------------------|\n                    authority               path\n                        |                    |\n        |-------------------------------||--------|\n  abc://username:password@example.com:123/path/data?key=value#fragid1\n  |-|   |---------------| |---------| |-|           |-------| |-----|\n   |            |              |       |                |        |\nscheme  user information     host     port            query   fragment\n\n  urn:example:mammal:monotreme:echidna\n  |-| |------------------------------|\n   |                 |\nscheme              path\n```\nWikipedia: [Uniform Resource Identifier](https://en.wikipedia.org/wiki/Uniform_Resource_Identifier)","ref":"Xema.Format.html#uri?/1","title":"Xema.Format.uri?/1","type":"function"},{"doc":"Checks if the `string` is a valid URI fragment representation.","ref":"Xema.Format.html#uri_fragment?/1","title":"Xema.Format.uri_fragment?/1","type":"function"},{"doc":"Checks if the `string` is a valid URI path representation.\n\nSee also `Xema.Format.uri?/1`.","ref":"Xema.Format.html#uri_path?/1","title":"Xema.Format.uri_path?/1","type":"function"},{"doc":"Checks if the `string` is a valid URI query representation.","ref":"Xema.Format.html#uri_query?/1","title":"Xema.Format.uri_query?/1","type":"function"},{"doc":"Checks if the `string` is a valid URI reference representation.","ref":"Xema.Format.html#uri_reference?/1","title":"Xema.Format.uri_reference?/1","type":"function"},{"doc":"Checks if the `string` is a valid URI template representation.","ref":"Xema.Format.html#uri_template?/1","title":"Xema.Format.uri_template?/1","type":"function"},{"doc":"Checks if the `string` is a valid URI template path representation.","ref":"Xema.Format.html#uri_template_path?/1","title":"Xema.Format.uri_template_path?/1","type":"function"},{"doc":"Checks if the `string` is a valid URI user info.\n\nSee also `Xema.Format.uri?/1`.","ref":"Xema.Format.html#uri_userinfo?/1","title":"Xema.Format.uri_userinfo?/1","type":"function"},{"doc":"The list of supported validators.","ref":"Xema.Format.html#t:format/0","title":"Xema.Format.format/0","type":"type"},{"doc":"Converts a JSON Schema to Xema source.","ref":"Xema.JsonSchema.html","title":"Xema.JsonSchema","type":"module"},{"doc":"This function converts a JSON Schema in Xema schema source. The argument\n`json_schema` is expected as a decoded JSON Schema.\n\nAll keys that are not standard JSON Schema keywords have to be known atoms. If\nthe schema has additional keys that are unknown atoms the option\n`atom: :force` is needed. In this case the atoms will be created. This is not\nneeded for keys expected by JSON Schema (e.g. in properties)\n\nOptions:\n* `:draft` specifies the draft to check the given JSON Schema. Possible values\n  are `\"draft4\"`, `\"draft6\"`, and `\"draft7\"`, default is `\"draft7\"`. If\n  `:draft` not set and the schema contains `$schema` then the value for\n  `$schema` is used for this option.\n* `:atom` creates atoms for unknown atoms when set to `:force`. This is just\n  needed for additional JSON Schema keywords.","ref":"Xema.JsonSchema.html#to_xema/2","title":"Xema.JsonSchema.to_xema/2","type":"function"},{"doc":"iex> Xema.JsonSchema.to_xema(%{\"type\" => \"integer\", \"minimum\" => 5})\n    {:integer, [minimum: 5]}\n\n    iex> schema = %{\n    ...>   \"type\" => \"object\",\n    ...>   \"properties\" => %{\"foo\" => %{\"type\" => \"integer\"}}\n    ...> }\n    iex> Xema.JsonSchema.to_xema(schema)\n    {:map, [keys: :strings, properties: %{\"foo\" => :integer}]}\n\n    iex> Xema.JsonSchema.to_xema(%{\"type\" => \"integer\", \"foo\" => \"bar\"}, atom: :force)\n    {:integer, [foo: \"bar\"]}","ref":"Xema.JsonSchema.html#to_xema/2-examples","title":"Examples - Xema.JsonSchema.to_xema/2","type":"function"},{"doc":"","ref":"Xema.JsonSchema.html#t:json_schema/0","title":"Xema.JsonSchema.json_schema/0","type":"type"},{"doc":"","ref":"Xema.JsonSchema.html#t:opts/0","title":"Xema.JsonSchema.opts/0","type":"type"},{"doc":"The behaviour for loaders.\n\nFor the loader configuration see \"[Configure a loader](loader.html)\".","ref":"Xema.Loader.html","title":"Xema.Loader","type":"behaviour"},{"doc":"","ref":"Xema.Loader.html#fetch/1","title":"Xema.Loader.fetch/1","type":"function"},{"doc":"This function expected an URI, to fetch the required data to create a schema.","ref":"Xema.Loader.html#c:fetch/1","title":"Xema.Loader.fetch/1","type":"callback"},{"doc":"The default loader.\n\nFor the loader configuration see \"[Configure a loader](loader.html)\".","ref":"Xema.NoLoader.html","title":"Xema.NoLoader","type":"module"},{"doc":"Returns for `any` data always the error tuple\n`{:error, \"No loader configured.\"}`.","ref":"Xema.NoLoader.html#fetch/1","title":"Xema.NoLoader.fetch/1","type":"function"},{"doc":"iex> Xema.NoLoader.fetch(nil)\n    {:error, \"No loader configured.\"}","ref":"Xema.NoLoader.html#fetch/1-examples","title":"Examples - Xema.NoLoader.fetch/1","type":"function"},{"doc":"This module contains a struct and functions to represent and handle\nreferences.","ref":"Xema.Ref.html","title":"Xema.Ref","type":"module"},{"doc":"Returns the schema and the root for the given `ref` and `xema`.","ref":"Xema.Ref.html#fetch!/3","title":"Xema.Ref.fetch!/3","type":"function"},{"doc":"","ref":"Xema.Ref.html#fragment/1","title":"Xema.Ref.fragment/1","type":"function"},{"doc":"Returns the reference key for a `Ref` or an `URI`.","ref":"Xema.Ref.html#key/1","title":"Xema.Ref.key/1","type":"function"},{"doc":"Creates a new reference from the given `pointer`.","ref":"Xema.Ref.html#new/1","title":"Xema.Ref.new/1","type":"function"},{"doc":"Creates a new reference from the given `pointer` and `uri`.","ref":"Xema.Ref.html#new/2","title":"Xema.Ref.new/2","type":"function"},{"doc":"","ref":"Xema.Ref.html#normalize/1","title":"Xema.Ref.normalize/1","type":"function"},{"doc":"Validates the given value with the referenced schema.","ref":"Xema.Ref.html#validate/3","title":"Xema.Ref.validate/3","type":"function"},{"doc":"A reference contains a `pointer` and an optional `uri`.","ref":"Xema.Ref.html#t:t/0","title":"Xema.Ref.t/0","type":"type"},{"doc":"This module contains the struct for the keywords of a schema.","ref":"Xema.Schema.html","title":"Xema.Schema","type":"module"},{"doc":"Fetches a subschema from the `schema` by the given `pointer`.\n\nIf `schema` contains the given pointer with a subschema, then `{:ok, schema}`\nis returned otherwise `:error`.","ref":"Xema.Schema.html#fetch/2","title":"Xema.Schema.fetch/2","type":"function"},{"doc":"Fetches a subschema from the `schema` by the given `pointer`.\n\nIf `schema` contains the given pointer with a subschema, then `{:ok, schema}`\nis returned otherwise a `SchemaError` is raised.","ref":"Xema.Schema.html#fetch!/2","title":"Xema.Schema.fetch!/2","type":"function"},{"doc":"Returns all keywords in a list.\n\nThe key `:data` is not a regular keyword and is not in the list.","ref":"Xema.Schema.html#keywords/0","title":"Xema.Schema.keywords/0","type":"function"},{"doc":"Returns a `%Schema{}` for the given `keywords` in the keyword list.","ref":"Xema.Schema.html#new/1","title":"Xema.Schema.new/1","type":"function"},{"doc":"Returns the `%Schema{}` as a map. Items which a `nil` value are not in the\nmap.","ref":"Xema.Schema.html#to_map/1","title":"Xema.Schema.to_map/1","type":"function"},{"doc":"Returns all available `type`s in a list.","ref":"Xema.Schema.html#types/0","title":"Xema.Schema.types/0","type":"function"},{"doc":"The struct contains the keywords for a schema.\n\n* `additional_items` disallow additional items, if set to false. The keyword\n  can also contain a schema to specify the type of additional items.\n* `additional_properties` disallow additional properties, if set to true.\n* 'all_of' a list of schemas they must all be valid.\n* 'any_of' a list of schemas with any valid schema.\n* `caster` a custom caster. This can be a function, a tuple with\n  module and function name, or a `Xema.Caster` behaviour.\n* `comment` for the schema.\n* `const` specifies a constant.\n* `content_encoding` annotation for the encoding.\n* `content_media_type` annotation for the media type.\n* `contains` validates a list whether any item is valid for the given schema.\n* `data` none schema data. Values in `data` will be interpreted as schemas\n  when possible. It is not recommended to put any data or schemas under this\n  key. The `data` property is mainly for compatibility with JsonSchema.\n* `default` this keyword can be used to supply a default value for JSON and\n  `defstruct`.\n* `definitions` contains schemas for reuse.\n* `dependencies` allows the schema of the map to change based on the presence\n  of certain special properties\n* `description` of the schema.\n* `else` see `if`, `then`, `else`.\n* `enum` specifies an enumeration\n* `examples` the value of this keyword must be an array. There are no\n  restrictions placed on the values within the array.\n* `exclusive_maximum` is a boolean. When true, it indicates that the range\n  excludes the maximum value.\n* `exclusive_minimum` is a boolean. When true, it indicates that the range\n  excludes the minimum value.\n* `format` semantic validation.\n* `id` a unique identifier.\n* `if`, `then`, `else`: These keywords work together to implement conditional\n  application of a subschema based on the outcome of another subschema.\n* `items` specifies the type(s) of the items.\n* `keys` could be `:atoms` or `:strings`.\n* `max_items` the maximum length of list.\n* `max_length` the maximum length of string.\n* `max_properties` the maximum count of properties for the map.\n* `maximum` the maximum value.\n* `min_items` the minimal length of list.\n* `min_length` the minimal length of string.\n* `min_properties` the minimal count of properties for the map.\n* `minimum` the minimum value.\n* `module` the module of a struct.\n* `multiple_of` is a number greater 0. The value has to be a multiple of this\n  number.\n* `not` negates the given schema\n* `one_of` the given data must be valid against exactly one of the given\n  subschemas.\n* `pattern_properties` specifies schemas for properties by patterns\n* `pattern` restrict a string to a particular regular expression.\n* `properties` specifies schemas for properties.\n* `property_names` a schema to check property names.\n* `ref` a reference to a schema.\n* `required` contains a set of required properties.\n* `schema` declares the used schema.\n* `title` of the schema.\n* `then` see `if`, `then`, `else`\n* `type` specifies the data type for a schema.\n* `unique_items` disallow duplicate items, if set to true.\n* `validator` a custom validator. This can be a function, a tuple with\n  module and function name, or a `Xema.Validator` behaviour.","ref":"Xema.Schema.html#t:t/0","title":"Xema.Schema.t/0","type":"type"},{"doc":"The `type` for the schema.","ref":"Xema.Schema.html#t:type/0","title":"Xema.Schema.type/0","type":"type"},{"doc":"","ref":"Xema.Schema.html#t:xema/0","title":"Xema.Schema.xema/0","type":"type"},{"doc":"A validator for the schema input.","ref":"Xema.SchemaValidator.html","title":"Xema.SchemaValidator","type":"module"},{"doc":"Returns `:ok` if the data valid against the xema schema.","ref":"Xema.SchemaValidator.html#validate!/1","title":"Xema.SchemaValidator.validate!/1","type":"function"},{"doc":"Some utilities for Xema.","ref":"Xema.Utils.html","title":"Xema.Utils","type":"module"},{"doc":"Returns whether the given `key` exists in the given `value`.\n\nReturns true if\n* `value` is a map and contains `key` as a key.\n* `value` is a keyword and contains `key` as a key.\n* `value` is a list of tuples with `key`as the first element.","ref":"Xema.Utils.html#has_key?/2","title":"Xema.Utils.has_key?/2","type":"function"},{"doc":"iex> alias Xema.Utils\n      iex> Utils.has_key?(%{foo: 5}, :foo)\n      true\n      iex> Utils.has_key?([foo: 5], :foo)\n      true\n      iex> Utils.has_key?([{\"foo\", 5}], \"foo\")\n      true\n      iex> Utils.has_key?([{\"foo\", 5}], \"bar\")\n      false\n      iex> Utils.has_key?([], \"bar\")\n      false","ref":"Xema.Utils.html#has_key?/2-examples","title":"Examples - Xema.Utils.has_key?/2","type":"function"},{"doc":"Returns the size of a `list` or `tuple`.","ref":"Xema.Utils.html#size/1","title":"Xema.Utils.size/1","type":"function"},{"doc":"Converts the given `string` to an existing atom. Returns `nil` if the\natom does not exist.","ref":"Xema.Utils.html#to_existing_atom/1","title":"Xema.Utils.to_existing_atom/1","type":"function"},{"doc":"iex> import Xema.Utils\n      iex> to_existing_atom(:my_atom)\n      :my_atom\n      iex> to_existing_atom(\"my_atom\")\n      :my_atom\n      iex> to_existing_atom(\"not_existing_atom\")\n      nil","ref":"Xema.Utils.html#to_existing_atom/1-examples","title":"Examples - Xema.Utils.to_existing_atom/1","type":"function"},{"doc":"Converts a `map` with integer keys or integer keys represented as strings to\na sorted list.\n\nReturns an ok tuple with the list or an `:error` atom.","ref":"Xema.Utils.html#to_sorted_list/2","title":"Xema.Utils.to_sorted_list/2","type":"function"},{"doc":"* `keys` - if `true` the resulting list has `{key, value}` items, with false\n             the list contains the values. Defaults to `[keys: true]`.","ref":"Xema.Utils.html#to_sorted_list/2-options","title":"Options: - Xema.Utils.to_sorted_list/2","type":"function"},{"doc":"iex> alias Xema.Utils\n      iex> Utils.to_sorted_list(%{2 => \"b\", 3 => \"c\", 1 => \"a\"})\n      {:ok, [{1, \"a\"}, {2, \"b\"}, {3, \"c\"}]}\n      iex> Utils.to_sorted_list(%{\"2\" => \"b\", \"3\" => \"c\", \"1\" => \"a\"})\n      {:ok, [{\"1\", \"a\"}, {\"2\", \"b\"}, {\"3\", \"c\"}]}\n      iex> Utils.to_sorted_list(%{\"2\" => \"b\", \"x\" => \"c\", \"1\" => \"a\"})\n      :error\n      iex> Utils.to_sorted_list(%{\"2\" => \"b\", \"3\" => \"c\", \"1\" => \"a\"}, keys: true)\n      {:ok, [{\"1\", \"a\"}, {\"2\", \"b\"}, {\"3\", \"c\"}]}\n      iex> Utils.to_sorted_list(%{\"2\" => \"b\", \"3\" => \"c\", \"1\" => \"a\"}, keys: false)\n      {:ok, [\"a\", \"b\", \"c\"]}","ref":"Xema.Utils.html#to_sorted_list/2-examples","title":"Examples - Xema.Utils.to_sorted_list/2","type":"function"},{"doc":"Returns `nil` if `uri_1` and `uri_2` are `nil`.\nParses a URI when the other URI is `nil`.\nMerges URIs if both are not nil.","ref":"Xema.Utils.html#update_uri/2","title":"Xema.Utils.update_uri/2","type":"function"},{"doc":"This module contains all validators to check data against a schema.","ref":"Xema.Validator.html","title":"Xema.Validator","type":"behaviour"},{"doc":"A callback for custom validators. For an example see:\n[Custom validators](examples.html#custom-validator)","ref":"Xema.Validator.html#c:validate/1","title":"Xema.Validator.validate/1","type":"callback"},{"doc":"Validates `data` against the given `schema`.","ref":"Xema.Validator.html#validate/2","title":"Xema.Validator.validate/2","type":"function"},{"doc":"","ref":"Xema.Validator.html#t:result/0","title":"Xema.Validator.result/0","type":"type"},{"doc":"Raised when a cast fails.","ref":"Xema.CastError.html","title":"Xema.CastError","type":"exception"},{"doc":"Formats the error map to an error message.","ref":"Xema.CastError.html#format_error/1","title":"Xema.CastError.format_error/1","type":"function"},{"doc":"","ref":"Xema.CastError.html#t:t/0","title":"Xema.CastError.t/0","type":"type"},{"doc":"Raised when a schema can't be build.","ref":"Xema.SchemaError.html","title":"Xema.SchemaError","type":"exception"},{"doc":"Raised when a validation fails.","ref":"Xema.ValidationError.html","title":"Xema.ValidationError","type":"exception"},{"doc":"This function returns an error message for an error or error tuple.","ref":"Xema.ValidationError.html#format_error/1","title":"Xema.ValidationError.format_error/1","type":"function"},{"doc":"```elixir\niex> schema = Xema.new(:integer)\niex> schema\n...>   |> Xema.Validator.validate(1.1)\n...>   |> Xema.ValidationError.format_error()\n\"Expected :integer, got 1.1.\"\n```","ref":"Xema.ValidationError.html#format_error/1-example","title":"Example - Xema.ValidationError.format_error/1","type":"function"},{"doc":"Traverse the error tree and invokes the given function.","ref":"Xema.ValidationError.html#travers_errors/4","title":"Xema.ValidationError.travers_errors/4","type":"function"},{"doc":"```elixir\niex> fun = fn _error, path, acc ->\n...>   [\"Error at \" <> inspect(path) | acc]\n...> end\niex>\niex> schema = Xema.new(\n...>   properties: %{\n...>     int: :integer,\n...>     names: {:list, items: :string},\n...>     num: [any_of: [:integer, :float]]\n...>   }\n...> )\niex>\niex> data = %{int: \"x\", names: [1, \"x\", 5], num: :foo}\niex>\niex> schema\n...>   |> Xema.Validator.validate(data)\n...>   |> Xema.ValidationError.travers_errors([], fun)\n[\n  \"Error at [:num]\",\n  \"Error at [:names, 2]\",\n  \"Error at [:names, 0]\",\n  \"Error at [:names]\",\n  \"Error at [:int]\",\n  \"Error at []\"\n]\n```","ref":"Xema.ValidationError.html#travers_errors/4-example","title":"Example - Xema.ValidationError.travers_errors/4","type":"function"},{"doc":"","ref":"Xema.ValidationError.html#t:opts/0","title":"Xema.ValidationError.opts/0","type":"type"},{"doc":"","ref":"Xema.ValidationError.html#t:path/0","title":"Xema.ValidationError.path/0","type":"type"},{"doc":"","ref":"Xema.ValidationError.html#t:t/0","title":"Xema.ValidationError.t/0","type":"type"}]}